/*-
 * Copyright (C) @BABOLO  2008 May 08
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef lint
static const char copyright[] = "\
@(#)Copyright (C) @BABOLO  2008 May 08\n\
@(#)All rights reserved.\n";
static const char rcsid[] = "$Id: memlook.c,v 1.1 2008/05/08 00:28:08 babolo Exp $";
#endif /* not lint */

#include <sys/types.h>
#include <sys/time.h>
#include <sysexits.h>
#include <unistd.h>
#include <stdlib.h>
#include <limits.h>
#include <stdio.h>
#include <err.h>
#include <babolo/BLINflag.h>

static void
usage(int ex) {
    fprintf(stderr, "memlook " VERS "  " DATE "\n"
"Usage:\n"
"memlook MemorySize (in Mbytes)\n"
);
    exit(ex);
}

int
main(int argc, char *argv[]) {
    struct timeval tv0, tv1;
    u_int32_t *mem, flags, errcnt;
    int i, ex = EX_OK;
    ssize_t msz;

    flags = BLIN_VER1;
    if  (!argv || !(argv[1]) || !(argv[1][0])) usage(EX_USAGE);
    msz = strtoul(argv[1], NULL, 0);
    ifBLIN_QV3(flags) fprintf(stderr, "%d\n", msz);
    if  (!msz) usage(EX_USAGE);
    mem = malloc(msz * 1024 * 1024);
    if  (!mem) {
        ifBLIN_QV1(flags) warnx("No mem");
        ex = EX_OSERR;
    } else {
        msz *= 1024 * 32;
        for (i = 0; i < msz; i++) {
            mem[i * 8 + 0] = mem[i * 8 + 1] = mem[i * 8 + 2] = mem[i * 8 + 3] = i;
            mem[i * 8 + 4] = mem[i * 8 + 5] = mem[i * 8 + 6] = mem[i * 8 + 7] = ~i;
        }
        for (errcnt = 0; !ex;) {
            sleep(300);
            if  (gettimeofday(&tv0, NULL)) {
                ifBLIN_QV1(flags) warn("gettimeofday #0");
                ex = EX_OSERR;
                break;
            }
            for (i = 0; i < msz; i++) {
                if  (  mem[i * 8 + 0] != i
                    || mem[i * 8 + 1] != i
                    || mem[i * 8 + 2] != i
                    || mem[i * 8 + 3] != i
                    ) {
                    printf( "@%d %08X%08X%08X%08X <> %08X%08X%08X%08X\n"
                          , 2 * i, i, i, i, i
                          , mem[i * 8 + 0], mem[i * 8 + 1], mem[i * 8 + 2], mem[i * 8 + 3]
                          );
                    mem[i * 8 + 0] = mem[i * 8 + 1] = mem[i * 8 + 2] = mem[i * 8 + 3] = i;
                    errcnt++;
                }
                if  (  mem[i * 8 + 4] != ~i
                    || mem[i * 8 + 5] != ~i
                    || mem[i * 8 + 6] != ~i
                    || mem[i * 8 + 7] != ~i
                    ) {
                    printf( "@%d %08X%08X%08X%08X <> %08X%08X%08X%08X\n"
                          , 2 * i + 1, ~i, ~i, ~i, ~i
                          , mem[i * 8 + 4], mem[i * 8 + 5], mem[i * 8 + 6], mem[i * 8 + 7]
                          );
                    mem[i * 8 + 4] = mem[i * 8 + 5] = mem[i * 8 + 6] = mem[i * 8 + 7] = ~i;
                    errcnt++;
            }   }
            if  (gettimeofday(&tv1, NULL)) {
                ifBLIN_QV1(flags) warn("gettimeofday #1");
                ex = EX_OSERR;
                break;
            }
            printf( "%d errors %ld.%06ld %s"
                  , errcnt
                  , (long)(tv1.tv_sec - tv0.tv_sec - ((tv1.tv_usec < tv0.tv_usec) ? 1 : 0))
                  , tv1.tv_usec - tv0.tv_usec + ((tv1.tv_usec < tv0.tv_usec) ? 1000000 : 0)
                  , ctime(&tv1.tv_sec)
                  );
    }   }
    exit(ex);
}
